// -----------------------------------------------------------------------------
//  Copyright (c) 2000     Southeastern Universities Research Association,
//                         Thomas Jefferson National Accelerator Facility
// 
//  This software was developed under a United States Government license
//  described in the NOTICE file included as part of this distribution.
// -----------------------------------------------------------------------------
// 
//  Project Overview: 
// 	This source code was developed as a component of the UnderLord 
// 	Scheduler for the Parallel Batch System - A joint, wide-area computing
// 	initiative at Jefferson Lab and MIT.
// 
//  Description:
// 	This header file defines the PBS_JobArray object which is used to 
//	maintain and return information from the PBS_ServerDatabase object.
//
//  Author:  Walt Akers
// -----------------------------------------------------------------------------
#ifndef _JOB_ARRAY_H_
#define _JOB_ARRAY_H_ 1

#include "PBS_JobCollection.h"

typedef int (*PBS_JobRecordSorter)(const PBS_JobRecord **, const PBS_JobRecord **);

// *****************************************************************************
// * class PBS_JobArray :
// *	This class is an extension of the PBS_JobCollection class.  Where 
// *	PBS_JobCollection optimizes the retrieval of jobs by their JobID, this
// *	class will use a secondary, array based approach to store the jobs
// *	an orderly fashion.  At initializartion, the user provides a sort 
// *	function that is used to order the jobs using qsort.
// *****************************************************************************
class PBS_JobArray : public PBS_JobCollection 
{
friend class PBS_ServerDatabase;

private:
	PBS_JobArray  ( const char * Name, size_t ArraySize=256 );
	~PBS_JobArray ( void );

protected:
	PBS_JobRecord ** records;
	size_t           arraySize;
	size_t           recordCnt;
	
	static int defaultSorter ( const PBS_JobRecord **, const PBS_JobRecord ** );
	
public:
	
	virtual void             clear     ( void );
	virtual void             sort      ( PBS_JobRecordSorter Sorter = PBS_JobArray::defaultSorter );
	virtual PBS_JobRecord  * addJob    ( PBS_JobRecord * job );
	virtual PBS_JobRecord  * delJob    ( char * job );
	virtual void             asciiDump ( FILE * fp = stdout, char * prefix = "\0");
	const   PBS_JobRecord  * operator[]( size_t idx );
};

#endif /* _JOB_ARRAY_H_ */
