// -----------------------------------------------------------------------------
//  Copyright (c) 2000     Southeastern Universities Research Association,
//                         Thomas Jefferson National Accelerator Facility
// 
//  This software was developed under a United States Government license
//  described in the NOTICE file included as part of this distribution.
// -----------------------------------------------------------------------------
// 
//  Project Overview: 
// 	This source code was developed as a component of the UnderLord 
// 	Scheduler for the Parallel Batch System - A joint, wide-area computing
// 	initiative at Jefferson Lab and MIT.
// 
//  Description:
// 	This file realizes the methods that are defined in the PBS_Log.h file.
//
//  Author:  Walt Akers
// -----------------------------------------------------------------------------
#include <PBS_Log.h>

extern "C" {
	#include <pbs_ifl.h>
	#include <sched_cmds.h>
	#include <log.h>
}
#include <stdlib.h>
#include <unistd.h>
#include <sys/time.h>
#include <time.h>

static int  UsePBSLogging = 0;
static char logMsg[1024];

const char * SetLogLocal ( char * szAppName )
	{
	static char * app = strdup("Application");
	
	UsePBSLogging = 0;
	if(szAppName && *szAppName && strcmp(szAppName, app))
		{
		free(app);
		app = strdup(szAppName);
		}

	return app;
	}
	
void SetLogRemote ( void )
	{
	UsePBSLogging = 1;
	}

void LogRecord(const char * local, const char * formatString, ...)
	{
	char * ptr = logMsg;

	va_list argp;
	va_start(argp, formatString);
	vsprintf(logMsg, formatString, argp);

	while((ptr=strchr(logMsg, '\n'))!=NULL) *ptr = ' ';
	
	if(UsePBSLogging)
		{
		log_record(PBSEVENT_SCHED, 0,  (char *)local, logMsg);
		}
	else	{
		struct timeval t;
		time_t now;

		gettimeofday(&t, NULL);
		now = t.tv_sec;
		fprintf(stdout, "%s Event: %s - %s", local, logMsg, ctime(&now));
		fflush(stdout);
		}
	va_end(argp);
	}
	

void LogError(const char * local, const char * formatString, ...)
	{
	char * ptr = logMsg;
	
	va_list argp;
	va_start(argp, formatString);
	vsprintf(logMsg, formatString, argp);

	while((ptr=strchr(logMsg, '\n'))!=NULL) *ptr = ' ';

	if(UsePBSLogging)
		{
		log_err(-1, (char *)local, logMsg);
		}
	else	{
		struct timeval t;
		time_t now;

		gettimeofday(&t, NULL);
		now = t.tv_sec;
		fprintf(stderr, "%s Error: %s - %s", local, logMsg, ctime(&now));
		fflush(stderr);
		}
	va_end(argp);
	}
