// -----------------------------------------------------------------------------
//  Copyright (c) 2000     Southeastern Universities Research Association,
//                         Thomas Jefferson National Accelerator Facility
// 
//  This software was developed under a United States Government license
//  described in the NOTICE file included as part of this distribution.
// -----------------------------------------------------------------------------
// 
//  Project Overview: 
// 	This source code was developed as a component of the UnderLord 
// 	Scheduler for the Parallel Batch System - A joint, wide-area computing
// 	initiative at Jefferson Lab and MIT.
// 
//  Description:
//	The header file JobCandidateSubSet.h contains the class definitions
// 	for the JobCandidateSubSet class.
// 
//  Author:  Walt Akers
// -----------------------------------------------------------------------------

#ifndef _JOB_CANDIDATE_SUB_SET_H_
#define _JOB_CANDIDATE_SUB_SET_H_ 1

#include <RangeDefinition.h>	// Needed for RangeDefintion class definition
#include <JobCandidate.h>	// Needed for inline JobCandidate methods

// *****************************************************************************
// * class JobCandidateSubSet :
// *	Because a job may provide an unlimited number of node specifications, 
// *	the JobCandidateSubSet class is provided to manage each of these
// *	specifications.  The JobCandidateSubSet class is a subclass of the
// *	RangeDefinition class and can therefore be passed to the ClusterNode
// *	object to obtain a list of time ranges when this job specification can
// *	be executed.
// *****************************************************************************
class JobCandidateSubSet : public RangeDefinition
{
protected:
	// *********************************************************************
	// * JobCandidate & owner:
	// *	This is the JobCandidate object that created and owns this
	// *	JobCandidateSubSet object. 
	// *********************************************************************
	JobCandidate & owner;

public:
	// *********************************************************************
	// * JobCandidateSubSet::JobCandidateSubSet :
	// *	This is the constructor for the object. It will copy the
	// *	reference to the JobCandidate object to the owner variable and
	// *	will then call the constructor for the RangeDefinition parent
	// *	class.
	// *********************************************************************
	JobCandidateSubSet ( JobCandidate & Owner, 
			     time_t Duration,
			     size_t NodeCnt, 
			     size_t ProcCnt, 
			     PBS_KeywordTable & keyTable)
		: owner          (Owner),
		  RangeDefinition(Duration, NodeCnt, ProcCnt, keyTable)
		{
		}
		
	// *********************************************************************
	// * JobCandidate::~JobCandidate :
	// *	This is the destructor for the object. 
	// *********************************************************************
	~JobCandidateSubSet ( void )
		{
		}	

	// *********************************************************************
	// * JobCandidateSubSet::getOwner :
	// *	This method allows the caller to obtain a reference to the 
	// *	owning JobCandidate object.
	// *********************************************************************
	JobCandidate & getOwner ( void )
		{
		return owner;
		}
};

#endif /* _JOB_CANDIDATE_SUB_SET_H_ */
