// -----------------------------------------------------------------------------
//  Copyright (c) 2000     Southeastern Universities Research Association,
//                         Thomas Jefferson National Accelerator Facility
// 
//  This software was developed under a United States Government license
//  described in the NOTICE file included as part of this distribution.
// -----------------------------------------------------------------------------
// 
//  Project Overview: 
// 	This source code was developed as a component of the UnderLord 
// 	Scheduler for the Parallel Batch System - A joint, wide-area computing
// 	initiative at Jefferson Lab and MIT.
// 
//  Description: This file contains the class definitions for a collection of
// 	objects that allow the scheduler to manage available nodes as a 
//	named set.  These sets will be used to allow jobs to be scheduled and
//	run on a restricted list of nodes.
// 
//  Author:  Walt Akers
// -----------------------------------------------------------------------------
#ifndef _NODE_SET_H_
#define _NODE_SET_H_ 1

#include <stdio.h>	// Neeeded for FILE definition
#include <PBS_Hash.h> 	// Needed for PBSStringHash

// *****************************************************************************
// * Class definitions for classes that are used in conjunction with the
// * NodeSetManager class.
// *****************************************************************************
class ClusterNode;
class NodeSet;


// *****************************************************************************
// * class NodeSetManager :
// *	This class is a container/distributor for named NodeSet objects. Each
// *	of the NodeSet objects corresponds to a list of ClusterNode objects 
// *	that a queue may use when scheduling/running jobs.
// *
// *	The NodeSetManager also contains an 'available' set - which contains 
// *	the list of all available nodes.  The 'available' set will be returned 
// *	to the caller when it requests a named node set that does not exist in 
// *	the list.
// *****************************************************************************
class NodeSetManager
{
protected:
	ClusterNode  ** available;
	size_t          availableCnt;
	NodeSet       * nodeSets;
	PBSStringHash   nodeSetHash;
	
public:
	NodeSetManager               ( void );
	~NodeSetManager              ( void );
	void           startCycle    ( ClusterNode ** Available, size_t NodeCnt );
	void           endCycle      ( void );
	ClusterNode ** getNodes      ( const char * name, size_t & cnt );
	void           asciiDump     ( FILE * fp = stdout, char * prefix = NULL );
};

#endif /* _NODE_DEF_H_ */
