#include <MatchMatrix.h>
#include <PBS_Server.h>
#include <unistd.h>
#include <JobDeployment.h>

const char * msg_daemonname = SetLogLocal("formatUsage");

void report ( PBS_Server & server, JobDeployment * scheduled );

int main (int argc, char ** argv)
{
int    connID = -1;
FILE * fp  = NULL;
char   buf[1024];
int balance = 0;

if(argc>1)
	{
	connID = pbs_connect(argv[1]);
	if(argc>2) balance = atoi(argv[2]); 
	}
else if	((fp = fopen("/usr/spool/PBS/server_name", "r"))!=NULL &&
	(fgets(buf, 1024, fp)!=NULL))
	{
	// Remove the trailing carriage return
	char * ptr;
	if((ptr = strchr(buf, '\n'))!=NULL) *ptr = 0;

	connID = pbs_connect(buf);
	fclose(fp);
	}

if(connID<=0)
	{
	fprintf(stdout, "Error: Server name unspecified or not valid\n");
	return 0;
	}

MatchMatrix matrix;
PBS_Server  server( connID );
PBS_Job **  jobs = server.getJobsByState(PBS_Job::QUEUED);
size_t      jobCnt = 0;


for(jobCnt=0; jobs && jobs[jobCnt]; jobCnt++);

 
matrix.startCycle(server);
SchedStatic::setBalanceFlag(balance);
fprintf(stdout, "RUNNING %s ALGORITHM\n", balance?"LOAD BALANCING":"NON-LOAD BALANCING");

char ** nodes = new char * [5];
nodes[0] = strdup("hpc5a");
nodes[1] = strdup("hpc6a");
nodes[2] = strdup("hpci1a");
nodes[3] = strdup("hpci2a");
nodes[4] = NULL;

time_t resStart = SchedStatic::getNow();
matrix.reserveTime(resStart+=3600, 3600, nodes, 1);
matrix.reserveTime(resStart+=3600, 3600, &(nodes[1]), 1);
matrix.reserveTime(resStart+=3600, 3600, &(nodes[2]), 1);
matrix.reserveTime(resStart+=3600, 3600, &(nodes[3]), 1);
matrix.reserveTime(resStart+=7200, 3600, &(nodes[1]), 3);

JobDeployment * scheduled = matrix.processJobs(jobs, jobCnt, 0);
if(scheduled!=NULL) report(server, scheduled);
matrix.endCycle();	
	
return 0;
}


void report ( PBS_Server & server, JobDeployment * scheduled )
{
PBSStringHash hash(256, 1);
PBS_NodeList & nodeList = (PBS_NodeList &)server;
size_t         nodeIdx  = 0;
size_t         maxIdx   = nodeList.getNodeCount();
PBS_Node    *  node     = NULL;
size_t         maxProcs = 0;

char buf[1024];

for(nodeIdx=0; nodeIdx<maxIdx; nodeIdx++)
	{
	const char * name   = nodeList[nodeIdx]->getName();
	size_t       nProcs = nodeList[nodeIdx]->getNumProcessors();
	
	for(int a=0; a<nProcs; a++)
		{
		sprintf(buf, "%s/%i", name, a); 
		hash.insert(buf, (void *)maxProcs++);
		}
	}

JobDeployment * jd = scheduled;
time_t lastStart = 0;
size_t   maxStarts = 16;
time_t * starts = (time_t *)malloc(maxStarts*sizeof(time_t));
size_t totalStarts = 0;


while(jd!=NULL)
	{
	if(jd->getStartTime()!=lastStart)
		{
		if(totalStarts>=maxStarts)
			{
			maxStarts*=2;
			starts = (time_t *) realloc(starts, maxStarts*sizeof(time_t));
			}
		starts[totalStarts++] = (lastStart=jd->getStartTime());
		}
	jd = jd->getNext();
	}

int * assignments = new int[totalStarts*maxProcs];
memset(assignments, 0, sizeof(int)*totalStarts*maxProcs);

jd = scheduled;
for(int c=0; c<totalStarts; c++)
	{
	while(jd && jd->getStartTime()<=starts[c])
		{
		int jobIndex = atoi(jd->getJob());
		const char ** ptr = jd->getProcs();
		for(int d=0; d<jd->getProcCnt(); d++)
			{
			int pIndex = (size_t)hash.find(ptr[d]);
			
			for(int e=c; e<totalStarts && jd->getEndTime()>=starts[e]; e++)
				{
				assignments[(e*maxProcs)+pIndex] = jobIndex;
				}
			}
		jd = jd->getNext();
		}
	}


for(int outIndex=0; outIndex<(totalStarts*maxProcs); outIndex++)
	{
	if((outIndex%maxProcs)==0) fprintf(stdout, " |\n%s :  |", SchedStatic::getTimeString(starts[outIndex/maxProcs]));
	else if((outIndex%4)==0 ) fprintf(stdout, " | ");
	if(assignments[outIndex]==0) fprintf(stdout, "____");
	else fprintf(stdout, "%4i", assignments[outIndex]);
	if(((outIndex+1)%4)!=0) fprintf(stdout, ", ");
	}
fprintf(stdout, " |\n");
}
